function marimba
%MARIMBA
% Marimba tuning demo

% Copyright 2010 Peter Fiala
% Laboratory of Acoustics
% Dept. of Telecommunications
% Budapest University of Technology and Economics

%% Build gui Figure with default parameters
MainFigure = figure('Visible', 'on',...
    'Name', 'Marimba Tuner', 'NumberTitle', 'off',...
    'Units', 'characters',...
    'Color', get(0,'defaultUicontrolBackgroundColor'), ...
    'Units', 'pixels',...
    'Position',[100 100 1000 700],...
    'MenuBar', 'none');

% GUI axes
for i = 5 : -1 : 1
    axes('Units', 'normalized', 'Tag', 'axMode', 'NextPlot', 'ReplaceChildren', ...
        'Position', [.55 .1+(i-1)*(.125+.025), .4, .125]);
end
axes('Units', 'normalized', 'Tag', 'axMesh', 'NextPlot', 'ReplaceChildren', ...
    'Position', [.55 .1+5*(.125+.025)+0.02, .4, .1]);
axes('Units', 'normalized', 'Tag', 'axSens', 'NextPlot', 'ReplaceChildren', ...
    'Position', [.05 .1, .45, .42]);

% GUI input panels
panelMater = uipanel(MainFigure, 'Title', 'Material Properies', ...
    'Position', [.05, .75, .22, .2]);
uidata = {
    'Young Modulus [Pa]', '1e10', 'E', @editMater_Callback;
    'Mass Density [kg/m3]', '830', 'Rho', @editMater_Callback;
    };
print_panel(MainFigure, panelMater, uidata);

panelGeom = uipanel(MainFigure, ...
    'Title', 'Geometry', ...
    'Position', [.28, .75, .22, .2]);
uidata = {
    'Marimba Length [m]', '1', 'L', @editGeom_Callback;
    'Marimba Width [m]', '1e-1', 'D', @editGeom_Callback;
    'Marimba Thickness [m]', '4e-2', 'H0', @editGeom_Callback;
    'Excavation Depth [-]', '0', 'H1', @editGeom_Callback;
    'Slope Start [-]', '0', 'L0', @editGeom_Callback;
    'Slope Length [-]', '0', 'L1', @editGeom_Callback;
    };
print_panel(MainFigure, panelGeom, uidata);

panelMesh = uipanel(MainFigure, ...
    'Title', 'Mesh', ...
    'Position', [.05, .63, .22, .2]);
uidata = {'Element Number [-]', '100', 'N', @editMesh_Callback};
print_panel(MainFigure, panelMesh, uidata);

panelCut = uipanel(MainFigure, ...
    'Title', 'Tuning Cut Parameters', ...
    'Position', [.05, .55, .22, .2]);
uidata = {
    'Cut depth [-]', '.5', 'Dh', @editCut_Callback;
    'Cut width [m]', '2e-2', 'Dl', @editCut_Callback;
    };
print_panel(MainFigure, panelCut, uidata);

% GUI buttons
uicontrol(MainFigure, 'Style', 'PushButton', 'Tag', 'butUpdate', ...
    'Units', 'Normalized', 'Position', [.05, .57, .09, .05],...
    'Callback', @MatGeoUpdate, 'String', 'Update Model');
uicontrol(MainFigure, 'Style', 'PushButton', 'Tag', 'butSens', ...
    'Units', 'Normalized', 'Position', [.15, .57, .09, .05],...
    'Callback', @Sensitivity, 'String', 'Sensitivity');
uicontrol(MainFigure, 'Style', 'PushButton', 'Tag', 'butChord', ...
    'Units', 'Normalized', 'Position', [.25, .57, .09, .05],...
    'Callback', @PlayChord, 'String', 'Play Chord');

%% initialize GUI by pressing Update button
MatGeoUpdate(MainFigure, []);

end

%%
function MatGeoUpdate(hObject, eventdata) %#ok<*INUSD>

handles = guihandles(hObject);
data = guidata(hObject);

%% Read Material parameters from GUI
mater = ReadMaterial(hObject);
geom = ReadGeometry(hObject);
mesh = ReadMesh(hObject);

%% Compute and store modes
h = tprofile(geom, mesh);
[data.Phi, data.Om] = beammodes(geom, mater, mesh, h, 10);
guidata(hObject, data);

%% Plot marimba model and modes
plot_fem_mesh(handles.axMesh, geom.L, mesh.N, h, @MarimbaHit);
plot_marimba_modes(handles.axMode, data.Phi, data.Om, @ModeHit);

end

%%
function ModeHit(hObject, eventdata, i) %#ok<*INUSL>

data = guidata(hObject);

Om = data.Om(i);
Phi = data.Phi(:,i);
F = zeros(size(Phi,1),1);
F(1) = 1;

fs = 44100;
T = 3;
U = marimba_synth(Phi, Om, F, fs, T);

soundsc(U, fs);

end

%%
function PlayChord(hObject, eventdata)

data = guidata(hObject);

Om = data.Om;
Phi = data.Phi;
F = zeros(size(Phi,1),1);
F(1) = 1;

fs = 44100;
T = 3;
f = [2 4 5 6] / 2;
U = marimba_synth(Phi, Om*f, F, fs, T);

soundsc(U, fs);

end

%%
function MarimbaHit(hObject, eventdata, i)

data = guidata(hObject);

Om = data.Om;
Phi = data.Phi;
F = zeros(size(Phi,1),1);
F(2*i+1) = 1;

fs = 44100;
T = 3;
U = marimba_synth(Phi, Om, F, fs, T);

soundsc(U, fs);

end

%%
function Sensitivity(hObject, eventdata)

handles = guihandles(hObject);
data = guidata(hObject);

MatGeoUpdate(hObject, eventdata);

%% Read GUI Data
mater = ReadMaterial(hObject);
geom = ReadGeometry(hObject);
mesh = ReadMesh(hObject);
cut = ReadCut(hObject);

%% Compute and plot sensitivity
nModes = 5;
Om = tuning_sensitivity(geom, mater, mesh, cut, tprofile(geom, mesh), nModes);
plot_tuning_sensitivity(handles.axSens, mesh, Om, data.Om(1:nModes),...
    @SensitivityButtonDownFcn);

end

%%
function SensitivityButtonDownFcn(hObject, eventdata, i)

handles = guihandles(hObject);
data = guidata(hObject);

%% Read GUI Data
mater = ReadMaterial(hObject);
geom = ReadGeometry(hObject);
mesh = ReadMesh(hObject);
cut = ReadCut(hObject);

%% Computed parameters
h = tprofile(geom, mesh);
q = ceil(cut.dL / (geom.L/mesh.N));
ind = round(i-1+(-q/2+1:q/2));
ind = ind(ind > 0);
ind = ind(ind <= mesh.N);
h(ind) = h(ind)*(1-cut.dh);
[data.Phi, data.Om] = beammodes(geom, mater, mesh, h, 10);

guidata(hObject, data);

%% Plot marimba model and modes
plot_fem_mesh(handles.axMesh, geom.L, mesh.N, h, @MarimbaHit);
plot_marimba_modes(handles.axMode, data.Phi, data.Om, @ModeHit);

end

%%
function mater = ReadMaterial(hObject)

handles = guihandles(hObject);
mater.E = str2double(get(handles.editE, 'String'));
mater.rho = str2double(get(handles.editRho, 'String'));

end

%%
function geom = ReadGeometry(hObject)

handles = guihandles(hObject);
geom.L = str2double(get(handles.editL, 'String'));
geom.d = str2double(get(handles.editD, 'String'));
geom.h0 = str2double(get(handles.editH0, 'String'));
geom.h1 = str2double(get(handles.editH1, 'String'));
geom.l0 = str2double(get(handles.editL0, 'String'));
geom.l1 = str2double(get(handles.editL1, 'String'));

end

%%
function mesh = ReadMesh(hObject)

handles = guihandles(hObject);
mesh.N = str2double(get(handles.editN, 'String'));

end

%%
function cut = ReadCut(hObject)

handles = guihandles(hObject);
cut.dL = str2double(get(handles.editDl, 'String'));
cut.dh = str2double(get(handles.editDh, 'String'));

end

%%
function print_panel(hObject, panel, uidata)

h = 1.5;
h0 = .7;
n = size(uidata,1);
Height = n*h+(n+2)*h0;
D = 30;
d = 10;

set(panel, 'Units', 'characters');
Pos = get(panel, 'Position');
Pos(2) = Pos(2)+Pos(4)-Height;
Pos(4) = Height;
set(panel, 'Position', Pos);

for i = 1 : size(uidata,1)
    uicontrol(panel, ...
        'Style', 'Text',...
        'Units', 'characters', ...
        'String', uidata{i,1},...
        'HorizontalAlignment', 'left',...
        'Position', [2*h0, Height-h0-i*(h+h0), D, h],...
        'Units', 'normalized');
    
    uicontrol(panel, ...
        'Style', 'Edit',...
        'Tag', sprintf('edit%s', uidata{i,3}),...
        'Units', 'characters', ...
        'Position', [Pos(3)-2*h0-d, Height-h0-i*(h+h0), d, h],...
        'Callback', uidata{i,4}, ...
        'BackGroundColor', [1 1 1], ...
        'String', uidata{i,2},...
        'Units', 'normalized');
end
set(panel, 'Units', 'normalized');

end

%%
function editMesh_Callback(hObject, eventdata)

uidata = {
    'N', [10 Inf], 100;
    };
checkdefault(hObject, uidata);

end

%%
function editMater_Callback(hObject, eventdata)

uidata = {
    'E', [0 Inf], 1e10;
    'Rho', [0 Inf], 830;
    };
checkdefault(hObject, uidata);

end

%%
function editGeom_Callback(hObject, eventdata)

uidata = {
    'L', [0 Inf], 1;
    'D', [0 Inf], .1;
    'H0', [0 Inf], 3e-2;
    'H1', [0 .99], 0;
    'L0', [0 .5], 0;
    'L1', [0 .5], 0;
    };
checkdefault(hObject, uidata);

end

%%
function editCut_Callback(hObject, eventdata)

uidata = {
    'Dh', [0 .99], .5;
    'Dl', [0 Inf], 2e-2;
    };
checkdefault(hObject, uidata);

end

%%
function checkdefault(hObject, data)

handles = guihandles(hObject);
for i = 1 : size(data,1)
    htag = sprintf('edit%s', data{i,1});
    lim = data{i,2};
    def = data{i,3};
    str = get(handles.(htag), 'String');
    num = str2double(str);
    if isnan(num)
        num = def;
    else
        if num < lim(1)
            num = lim(1);
        end
        if num > lim(2)
            num = lim(2);
        end
    end
    set(handles.(htag), 'String', num2str(num, '%g'));
end

end